@@ -7,11 +7,30 @@
 /* Mario Bezzi. Thanks Mario! Also special thanks to Greg Smith who           */
 /* introduced iregs, needed when a page fault occurs.                         */
 /*                                                                            */
-/*                              (c) Copyright Bernard van der Helm, 2000-2006 */
+/*                              (c) Copyright Bernard van der Helm, 2000-2007 */
 /*                              Noordwijkerhout, The Netherlands.             */
 /*                                                                            */
 /*----------------------------------------------------------------------------*/
 
+// $Id: cmpsc.c,v 1.51 2007/06/23 00:04:04 ivan Exp $
+//
+// $Log: cmpsc.c,v $
+// Revision 1.51  2007/06/23 00:04:04  ivan
+// Update copyright notices to include current year (2007)
+//
+// Revision 1.50  2007/02/10 09:14:32  bernard
+// Decompression performance patch
+//
+// Revision 1.49  2007/01/13 07:11:45  bernard
+// backout ccmask
+//
+// Revision 1.48  2007/01/12 15:21:12  bernard
+// ccmask phase 1
+//
+// Revision 1.47  2006/12/08 09:43:18  jj
+// Add CVS message log
+//
+
 #include "hstdinc.h"
 
 #if !defined(_HENGINE_DLL_)
@@ -309,7 +328,7 @@ static void ARCH_DEP(print_sd)(int r2, REGS *regs, BYTE *sd, int index);
 #if !defined(TESTCH261)
 #define TESTCH261(regs, processed, length) \
 {\
-  if(((processed) += (length)) > 260) \
+  if(unlikely(((processed) += (length)) > 260)) \
     ARCH_DEP(program_interrupt)((regs), PGM_DATA_EXCEPTION); \
 }
 #endif /* !defined(TESTCH261) */
@@ -369,7 +388,7 @@ static void ARCH_DEP(compress)(int r1, int r2, REGS *regs, REGS *iregs)
   BYTE cce[8];                         /* compression character entry         */
   int eos;                             /* indication end of source            */
   U16 last_match;                      /* Last matched index symbol           */
-  BYTE next_ch;                        /* next character read                 */
+  BYTE next_ch=0;                      /* next character read                 */
   int xlated;                          /* number of bytes processed           */
 
   /* Initialize end of source */
@@ -381,14 +400,14 @@ static void ARCH_DEP(compress)(int r1, int r2, REGS *regs, REGS *iregs)
   {
 
     /* Can we write an index or interchange symbol */
-    if(((GR1_cbn(iregs) + GR0_smbsz(regs) - 1) / 8) >= GR_A(r1 + 1, iregs))
+    if(unlikely(((GR1_cbn(iregs) + GR0_smbsz(regs) - 1) / 8) >= GR_A(r1 + 1, iregs)))
     {
       regs->psw.cc = 1;
       return;
     }
 
     /* Get the next character, return on end of source */
-    if(ARCH_DEP(fetch_ch)(r2, regs, iregs, &next_ch, 0))
+    if(unlikely(ARCH_DEP(fetch_ch)(r2, regs, iregs, &next_ch, 0)))
       return;
 
     /* Get the alphabet entry */
@@ -434,7 +453,7 @@ static void ARCH_DEP(compress)(int r1, int r2, REGS *regs, REGS *iregs)
     COMMITREGS(regs, iregs, r1, r2);
 
     /* When reached end of source, return to caller */
-    if(eos)
+    if(unlikely(eos))
       return;
   }
 
@@ -448,7 +467,7 @@ static void ARCH_DEP(compress)(int r1, int r2, REGS *regs, REGS *iregs)
 static void ARCH_DEP(expand)(int r1, int r2, REGS *regs, REGS *iregs)
 {
   BYTE byte;                           /* a byte                              */
-  U16 index_symbol;                    /* Index symbol                        */
+  U16 index_symbol=0;                  /* Index symbol                        */
   BYTE ece[8];                         /* Expansion Character Entry           */
   int entries;                         /* Entries processed                   */
   U16 pptr;                            /* predecessor pointer                 */
@@ -461,7 +480,7 @@ static void ARCH_DEP(expand)(int r1, int r2, REGS *regs, REGS *iregs)
   {
 
     /* Get an index symbol, return on end of source */
-    if(ARCH_DEP(fetch_is)(r2, regs, iregs, &index_symbol))
+    if(unlikely(ARCH_DEP(fetch_is)(r2, regs, iregs, &index_symbol)))
       return;
 
     /* Check if this is an alphabet entry */
@@ -470,7 +489,7 @@ static void ARCH_DEP(expand)(int r1, int r2, REGS *regs, REGS *iregs)
 
       /* Write the alphabet entry, return on trouble */
       byte = index_symbol;
-      if(ARCH_DEP(store_ch)(r1, regs, iregs, &byte, 1, 0))
+      if(unlikely(ARCH_DEP(store_ch)(r1, regs, iregs, &byte, 1, 0)))
         return;
 
       /* Adjust destination registers */
@@ -499,7 +518,7 @@ static void ARCH_DEP(expand)(int r1, int r2, REGS *regs, REGS *iregs)
         TESTCH261(regs, written, ECE_psl(ece));
 
         /* Output extension characters in preceeded entry, return on trouble */
-        if(ARCH_DEP(store_ch)(r1, regs, iregs, ECE_ec(ece), ECE_psl(ece), ECE_ofst(ece)))
+        if(unlikely(ARCH_DEP(store_ch)(r1, regs, iregs, ECE_ec(ece), ECE_psl(ece), ECE_ofst(ece))))
           return;
 
         /* Get the preceeding entry */
@@ -507,7 +526,7 @@ static void ARCH_DEP(expand)(int r1, int r2, REGS *regs, REGS *iregs)
         FETCH_ECE(r2, regs, ece, pptr);
 
         /* Check for processing entry 128 */
-        if(++entries > 127)
+        if(unlikely(++entries > 127))
         {
 
 #if defined(OPTION_CMPSC_DEBUGLVL) && OPTION_CMPSC_DEBUGLVL & 2
@@ -522,7 +541,7 @@ static void ARCH_DEP(expand)(int r1, int r2, REGS *regs, REGS *iregs)
       TESTCH261(regs, written, ECE_csl(ece));
 
       /* Output extension characters in last or only unpreceeded entry, return on trouble */
-      if(ARCH_DEP(store_ch)(r1, regs, iregs, ECE_ec(ece), ECE_csl(ece), 0))
+      if(unlikely(ARCH_DEP(store_ch)(r1, regs, iregs, ECE_ec(ece), ECE_csl(ece), 0)))
         return;
 
       /* Adjust destination registers */
@@ -589,7 +608,7 @@ static void ARCH_DEP(print_cce)(int r2, REGS *regs, BYTE *cce, int index)
 static int ARCH_DEP(fetch_ch)(int r2, REGS *regs, REGS *iregs, BYTE *ch, int offset)
 {
   /* Check for end of source condition */
-  if(GR_A(r2 + 1, iregs) <= (U32) offset)
+  if(unlikely(GR_A(r2 + 1, iregs) <= (U32) offset))
   {
 
 #if defined(OPTION_CMPSC_DEBUGLVL) && OPTION_CMPSC_DEBUGLVL & 1
@@ -662,7 +681,7 @@ static int ARCH_DEP(fetch_is)(int r2, REGS *regs, REGS *iregs, U16 *index_symbol
   BYTE work[3];
 
   /* Check if we can read an index symbol */
-  if(((GR1_cbn(iregs) + GR0_smbsz(regs) - 1) / 8) >= GR_A(r2 + 1, iregs))
+  if(unlikely(((GR1_cbn(iregs) + GR0_smbsz(regs) - 1) / 8) >= GR_A(r2 + 1, iregs)))
   {
 
 #if defined(OPTION_CMPSC_DEBUGLVL) && OPTION_CMPSC_DEBUGLVL & 1
@@ -673,8 +692,8 @@ static int ARCH_DEP(fetch_is)(int r2, REGS *regs, REGS *iregs, U16 *index_symbol
     return(1);
   }
 
-  /* Get the storage */
-  memset(work, 0, 3);
+  /* Clear possible fetched 3rd byte */
+  work[2] = 0;
   ARCH_DEP(vfetchc)(&work, (GR0_smbsz(regs) + GR1_cbn(iregs) - 1) / 8, GR_A(r2, iregs) & ADDRESS_MAXWRAP(regs), r2, regs);
 
   /* Get the bits */
@@ -773,7 +792,7 @@ static enum cmpsc_status ARCH_DEP(search_cce)(int r2, REGS *regs, REGS *iregs, B
   ind_search_siblings = 1;
 
   /* Get the next character when there are children */
-  if(CCE_ccs(cce) && ARCH_DEP(fetch_ch)(r2, regs, iregs, next_ch, 0))
+  if(unlikely((CCE_ccs(cce) && ARCH_DEP(fetch_ch)(r2, regs, iregs, next_ch, 0))))
     return(end_of_source);
 
   /* Now check all children in parent */
@@ -792,7 +811,7 @@ static enum cmpsc_status ARCH_DEP(search_cce)(int r2, REGS *regs, REGS *iregs, B
       ind_search_siblings = 0;
 
       /* Check if child should not be examined */
-      if(!CCE_x(cce, i))
+      if(unlikely(!CCE_x(cce, i)))
       {
 
         /* No need to examine child, found the last match */
@@ -872,7 +891,7 @@ static enum cmpsc_status ARCH_DEP(search_sd)(int r2, REGS *regs, REGS *iregs, BY
     {
 
       /* Stop searching when child tested and no consecutive child character */
-      if(!ind_search_siblings && !SD_ccc(regs, sd, i))
+      if(unlikely(!ind_search_siblings && !SD_ccc(regs, sd, i)))
         return(write_index_symbol);
 
       if(*next_ch == SD_sc(regs, sd, i))
@@ -882,7 +901,7 @@ static enum cmpsc_status ARCH_DEP(search_sd)(int r2, REGS *regs, REGS *iregs, BY
         ind_search_siblings = 0;
 
         /* Check if child should not be examined */
-        if(!SD_ecb(regs, sd, i, cce, y_in_parent))
+        if(unlikely(!SD_ecb(regs, sd, i, cce, y_in_parent)))
         {
 
           /* No need to examine child, found the last match */
@@ -938,7 +957,7 @@ static int ARCH_DEP(store_ch)(int r1, REGS *regs, REGS *iregs, BYTE *data, int l
 {
 
   /* Check destination size */
-  if(GR_A(r1 + 1, iregs) < length + (U32) offset)
+  if(unlikely(GR_A(r1 + 1, iregs) < length + (U32) offset))
   {
 
 #if defined(OPTION_CMPSC_DEBUGLVL) && OPTION_CMPSC_DEBUGLVL & 2
@@ -978,7 +997,7 @@ static void ARCH_DEP(store_is)(int r1, int r2, REGS *regs, REGS *iregs, U16 inde
   BYTE work[3];                        /* work bytes                          */
 
   /* Check if symbol translation is requested */
-  if(GR0_st(regs))
+  if(unlikely(GR0_st(regs)))
   {
 
     /* Get the interchange symbol */
@@ -1002,7 +1021,7 @@ static void ARCH_DEP(store_is)(int r1, int r2, REGS *regs, REGS *iregs, U16 inde
   threebytes = (GR0_smbsz(regs) + GR1_cbn(iregs)) > 16;
 
   /* Get the storage */
-  if(threebytes)
+  if(unlikely(threebytes))
     ARCH_DEP(vfetchc)(work, 2, GR_A(r1, iregs) & ADDRESS_MAXWRAP(regs), r1, regs);
   else
     ARCH_DEP(vfetchc)(work, 1, GR_A(r1, iregs) & ADDRESS_MAXWRAP(regs), r1, regs);
@@ -1014,7 +1033,7 @@ static void ARCH_DEP(store_is)(int r1, int r2, REGS *regs, REGS *iregs, U16 inde
   work[1] |= (set_mask >> 8) & 0xFF;
 
   /* Set the storage */
-  if(threebytes)
+  if(unlikely(threebytes))
   {
     work[2] &= clear_mask & 0xFF;
     work[2] |= set_mask & 0xFF;
@@ -1089,7 +1108,7 @@ DEF_INST(compression_call)
 #endif /* OPTION_CMPSC_DEBUGLVL */
 
   /* Check the registers on even-odd pairs and valid compression-data symbol size */
-  if(r1 & 0x01 || r2 & 0x01 || !GR0_cdss(regs) || GR0_cdss(regs) > 5)
+  if(unlikely(r1 & 0x01 || r2 & 0x01 || !GR0_cdss(regs) || GR0_cdss(regs) > 5))
     ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
 
 #if (__GEN_ARCH == 900)
@@ -1101,7 +1120,7 @@ DEF_INST(compression_call)
   COMMITREGS(&iregs, regs, r1, r2);
 
   /* Now go to the requested function */
-  if(GR0_e(regs))
+  if(likely(GR0_e(regs)))
     ARCH_DEP(expand)(r1, r2, regs, &iregs);
   else
     ARCH_DEP(compress)(r1, r2, regs, &iregs);
