@@ -1,3 +1,13 @@
+// $Id: cpuint.h,v 1.39 2006/12/21 22:39:38 gsmith Exp $
+//
+// $Log: cpuint.h,v $
+// Revision 1.39  2006/12/21 22:39:38  gsmith
+// 21 Dec 2006 Range for s+, t+ - Greg Smith
+//
+// Revision 1.38  2006/12/08 09:43:19  jj
+// Add CVS message log
+//
+
 /**********************************************************************
  Interrupts_State & Interrupts_Mask bits definition (Initial_Mask=800E)
  Machine check, PER and external interrupt subclass bit positions
@@ -5,12 +15,12 @@
  Floating interrupts are made pending to all CPUs, and are 
  recorded in the sysblk structure, CPU specific interrupts
  are recorded in the regs structure.
- hi0m mmmm pppp p00p xxxx xxxx xxxx hhhs : type U32
- || | |||| |||| |--| |||| |||| |||| ||||   h:mask  is always '1'
- || | |||| |||| |  | |||| |||| |||| ||||   s:state is always '1'
- || | |||| |||| |  | |||| |||| |||| |||+--> '1' : PSW_WAIT
- || | |||| |||| |  | |||| |||| |||| ||+---> '1' : RESTART
- || | |||| |||| |  | |||| |||| |||| |+----> '1' : BROADCAST
+ hi0m mmmm pppp p00p xxxx xxxx xxxx h0hs : type U32
+ || | |||| |||| |--| |||| |||| |||| |-||   h:mask  is always '1'
+ || | |||| |||| |  | |||| |||| |||| | ||   s:state is always '1'
+ || | |||| |||| |  | |||| |||| |||| | |+--> '1' : PSW_WAIT
+ || | |||| |||| |  | |||| |||| |||| | +---> '1' : RESTART
+ || | |||| |||| |  | |||| |||| |||| |             (available)
  || | |||| |||| |  | |||| |||| |||| +-----> '1' : STORSTAT
  || | |||| |||| |  | |||| |||| ||||
  || | |||| |||| |  | |||| |||| |||+-------> '1' : ETR
@@ -50,7 +60,7 @@
 **********************************************************************/
 
 // Hercules internal bit# macro: bit numbers are referenced counting
-// from the RIGHT to left (i.e. 32 <-- 0) and thus the numerical value
+// from the RIGHT to left (i.e. 31 <-- 0) and thus the numerical value
 // of a set bit is equal to two raised to the power of the bit position.
 
 // While this is the COMPLETE OPPOSITE from the way bit numbers are
@@ -94,7 +104,7 @@
 #define IC_EXTSIG           5 /* 0x00000020 - Architecture dependent (CR0) */
 #define IC_ETR              4 /* 0x00000010 - Architecture dependent (CR0) */
 #define IC_STORSTAT         3 /* 0x00000008 */
-#define IC_BROADCAST        2 /* 0x00000004 */
+#define IC_UNUSED_2         2 /* 0x00000004 */
 #define IC_RESTART          1 /* 0x00000002 */
 #define IC_PSW_WAIT         0 /* 0x00000001 */
 
@@ -102,7 +112,6 @@
 #define IC_INITIAL_STATE   BIT(IC_PSW_WAIT)
 #define IC_INITIAL_MASK  ( BIT(IC_INTERRUPT) \
                          | BIT(IC_RESTART) \
-                         | BIT(IC_BROADCAST) \
                          | BIT(IC_STORSTAT) \
                          )
 
@@ -112,7 +121,26 @@
 /* I/O interrupt subclasses */
 #define IC_IOPENDING     ( BIT(IC_IO) )
 
+/* External interrupt subclasses in CR 0 */
+#define IC_EXT_SCM_CR0   ( BIT(IC_MALFALT) \
+                         | BIT(IC_EMERSIG) \
+                         | BIT(IC_EXTCALL) \
+                         | BIT(IC_TODSYNC) \
+                         | BIT(IC_CLKC) \
+                         | BIT(IC_PTIMER) \
+                         | BIT(IC_SERVSIG) \
+                         | BIT(IC_ITIMER) \
+                         | BIT(IC_INTKEY) \
+                         | BIT(IC_EXTSIG) \
+                         | BIT(IC_ETR) \
+                         )
+
 /* External interrupt subclasses */
+
+/* 
+ * Adds ECPS:VM Vtimer which has no individual
+ * subclass mask in CR0
+ */
 #define IC_EXTPENDING    ( BIT(IC_MALFALT) \
                          | BIT(IC_EMERSIG) \
                          | BIT(IC_EXTCALL) \
@@ -150,8 +178,7 @@
                          )
 
 /* SIE & Assist supported events */
-#define IC_SIE_INT       ( BIT(IC_BROADCAST) \
-                         | BIT(IC_IO) \
+#define IC_SIE_INT       ( BIT(IC_IO) \
                          | BIT(IC_CLKC) \
                          | BIT(IC_PTIMER) \
                          | BIT(IC_ITIMER) \
@@ -166,6 +193,19 @@
  * to avoid multiple updates to `ints_mask'.
  */
 
+#undef IC_CR0_TO_INTMASK
+#if defined(FEATURE_ECPSVM)
+
+#define IC_CR0_TO_INTMASK(_regs) \
+(  ( (_regs)->CR(0) & IC_EXT_SCM_CR0) \
+  | (((_regs)->CR(0) & BIT(IC_ITIMER)) ? BIT(IC_ECPSVTIMER) : 0) )
+#else
+
+#define IC_CR0_TO_INTMASK(_regs) \
+ ( (_regs)->CR(0) & IC_EXT_SCM_CR0)
+
+#endif /* FEATURE_ECPSVM */
+
 #define IC_MASK(_regs) \
  ( ( IC_INITIAL_MASK ) \
  | ( ECMODE(&(_regs)->psw) \
@@ -173,8 +213,8 @@
      : ( ((_regs)->psw.sysmask & 0xFE) ? BIT(IC_IO) : 0 ) \
    ) \
  | ( MACHMASK(&(_regs)->psw) ? ((_regs)->CR(14) & IC_MCKPENDING) : 0 ) \
- | ( PER_MODE(_regs) ? (((_regs)->CR(9) >> IC_CR9_SHIFT) & IC_PER_MASK) : 0 ) \
- | ( ((_regs)->psw.sysmask & PSW_EXTMASK) ? ((_regs)->CR(0) & IC_EXTPENDING) : 0 ) \
+ | ( PER_MODE((_regs)) ? ((_regs)->ints_mask & IC_PER_MASK) : 0 ) \
+ | ( ((_regs)->psw.sysmask & PSW_EXTMASK) ? (IC_CR0_TO_INTMASK((_regs))) : 0 ) \
  | ( WAITSTATE(&(_regs)->psw) ? BIT(IC_PSW_WAIT) : 0 ) \
  )
 
@@ -182,8 +222,8 @@
  ( ( IC_INITIAL_MASK ) \
  | ( ((_regs)->psw.sysmask & PSW_IOMASK) ? BIT(IC_IO) : 0 ) \
  | ( MACHMASK(&(_regs)->psw) ? ((_regs)->CR(14) & IC_MCKPENDING) : 0 ) \
- | ( PER_MODE(_regs) ? (((_regs)->CR(9) >> IC_CR9_SHIFT) & IC_PER_MASK) : 0 ) \
- | ( ((_regs)->psw.sysmask & PSW_EXTMASK) ? ((_regs)->CR(0) & IC_EXTPENDING) : 0 ) \
+ | ( PER_MODE((_regs)) ? ((_regs)->ints_mask & IC_PER_MASK) : 0 ) \
+ | ( ((_regs)->psw.sysmask & PSW_EXTMASK) ? (IC_CR0_TO_INTMASK((_regs))) : 0 ) \
  | ( WAITSTATE(&(_regs)->psw) ? BIT(IC_PSW_WAIT) : 0 ) \
  )
 
@@ -191,21 +231,24 @@
  ( ( IC_INITIAL_MASK ) \
  | ( ((_regs)->psw.sysmask & 0xFE) ? BIT(IC_IO) : 0 ) \
  | ( MACHMASK(&(_regs)->psw) ? ((_regs)->CR(14) & IC_MCKPENDING) : 0 ) \
- | ( PER_MODE(_regs) ? (((_regs)->CR(9) >> IC_CR9_SHIFT) & IC_PER_MASK) : 0 ) \
- | ( ((_regs)->psw.sysmask & PSW_EXTMASK) ? ((_regs)->CR(0) & IC_EXTPENDING) : 0 ) \
+ | ( PER_MODE((_regs)) ? ((_regs)->ints_mask & IC_PER_MASK) : 0 ) \
+ | ( ((_regs)->psw.sysmask & PSW_EXTMASK) ? (IC_CR0_TO_INTMASK((_regs))) : 0 ) \
  | ( WAITSTATE(&(_regs)->psw) ? BIT(IC_PSW_WAIT) : 0 ) \
  )
 
+/* Note: if PER mode, invalidate the AIA to force instfetch to be called */
 #define SET_IC_ECMODE_MASK(_regs) \
  do { \
    (_regs)->ints_mask = IC_ECMODE_MASK((_regs)); \
-   (_regs)->permode = ((IC_PER_MASK & (_regs)->ints_mask) != 0); \
+   if ( ( (_regs)->permode = PER_MODE((_regs)) ) ) \
+    INVALIDATE_AIA((_regs)); \
  } while (0)
 
 #define SET_IC_BCMODE_MASK(_regs) \
  do { \
    (_regs)->ints_mask = IC_BCMODE_MASK((_regs)); \
-   (_regs)->permode = ((IC_PER_MASK & (_regs)->ints_mask) != 0); \
+   if ( ( (_regs)->permode = PER_MODE((_regs)) ) ) \
+    INVALIDATE_AIA((_regs)); \
  } while (0)
 
 #undef SET_IC_MASK
@@ -213,33 +256,37 @@
  #define SET_IC_MASK(_regs) \
   do { \
     (_regs)->ints_mask = IC_MASK((_regs)); \
-    (_regs)->permode = ((IC_PER_MASK & (_regs)->ints_mask) != 0); \
+    if ( ( (_regs)->permode = PER_MODE((_regs)) ) ) \
+     INVALIDATE_AIA((_regs)); \
   } while (0)
 #else
  #define SET_IC_MASK(_regs) SET_IC_ECMODE_MASK(_regs)
 #endif
 
-/* State bits indicate what interrupts are possibly pending
+/*
+ * State bits indicate what interrupts are possibly pending
  * for a CPU.  These bits can be set by any thread and therefore
- * are serialized by the `intlock'.  An exception is for PER
- * events.  Updates then use macros that provide serialization
- * on supported architectures.
+ * are serialized by the `intlock'.
+ * For PER, the state bits are set when CR9 is loaded and the mask
+ * bits are set when a PER event occurs
  */
 
 #define SET_IC_TRACE \
  do { \
    int i; \
-   int tracing = (sysblk.instbreak || sysblk.inststep || sysblk.insttrace); \
    U32 mask = sysblk.started_mask; \
    for (i = 0; mask; i++) { \
-     if (mask & 1) { \
-       sysblk.regs[i]->tracing = tracing; \
+     if (mask & 1) \
        sysblk.regs[i]->ints_state |= BIT(IC_INTERRUPT); \
-     } \
      mask >>= 1; \
    } \
  } while (0)
 
+#define SET_IC_PER(_regs) \
+ do { \
+  (_regs)->ints_state |= (((_regs)->CR(9) >> IC_CR9_SHIFT) & IC_PER_MASK); \
+  (_regs)->ints_mask  &= (~IC_PER_MASK | (_regs)->ints_state); \
+ } while (0)
 
   /* * * * * * * * * * * * * *
    * Set state bit to '1'    *
@@ -255,11 +302,6 @@
    (_regs)->ints_state |= BIT(IC_INTERRUPT) | BIT(IC_RESTART); \
  } while (0)
 
-#define ON_IC_BROADCAST(_regs) \
- do { \
-   (_regs)->ints_state |= BIT(IC_INTERRUPT) | BIT(IC_BROADCAST); \
- } while (0)
-
 #define ON_IC_STORSTAT(_regs) \
  do { \
    (_regs)->ints_state |= BIT(IC_INTERRUPT) | BIT(IC_STORSTAT); \
@@ -393,45 +435,40 @@
      (_regs)->ints_state |= BIT(IC_EMERSIG); \
  } while (0)
 
-    /* PER bits are a little different in that we only set the
-     * state bit to one if the mask bit is on.  There is no need
-     * to set the IC_INTERRUPT bit.
+    /*
+     * When a PER event occurs we set the bit in ints_mask instead of
+     * ints_state; therefore intlock does not need to be held.
+     * The ints_state bits are set when CR9 is loaded.
      */
 
 #define ON_IC_PER_SB(_regs) \
  do { \
-   if ( (_regs)->ints_mask & BIT(IC_PER_SB) ) \
-     (_regs)->ints_state |= BIT(IC_PER_SB); \
+   (_regs)->ints_mask |= BIT(IC_PER_SB); \
  } while (0)
 
 #define ON_IC_PER_IF(_regs) \
  do { \
-   if ( (_regs)->ints_mask & BIT(IC_PER_IF) ) \
-     (_regs)->ints_state |= BIT(IC_PER_IF); \
+   (_regs)->ints_mask |= BIT(IC_PER_IF); \
  } while (0)
 
 #define ON_IC_PER_SA(_regs) \
  do { \
-   if ( (_regs)->ints_mask & BIT(IC_PER_SA) ) \
-     (_regs)->ints_state |= BIT(IC_PER_SA); \
+     (_regs)->ints_mask |= BIT(IC_PER_SA); \
  } while (0)
 
 #define ON_IC_PER_GRA(_regs) \
  do { \
-   if ( (_regs)->ints_mask & BIT(IC_PER_GRA) ) \
-     (_regs)->ints_state |= BIT(IC_PER_GRA); \
+     (_regs)->ints_mask |= BIT(IC_PER_GRA); \
  } while (0)
 
 #define ON_IC_PER_STURA(_regs) \
  do { \
-   if ( (_regs)->ints_mask & BIT(IC_PER_STURA) ) \
-     (_regs)->ints_state |= BIT(IC_PER_STURA); \
+     (_regs)->ints_mask |= BIT(IC_PER_STURA); \
  } while (0)
 
 #define ON_IC_PER_IFNUL(_regs) \
  do { \
-   if ( (_regs)->ints_mask & BIT(IC_PER_IFNUL) ) \
-     (_regs)->ints_state |= BIT(IC_PER_IFNUL); \
+   (_regs)->ints_mask |= BIT(IC_PER_IFNUL); \
  } while (0)
 
 
@@ -449,11 +486,6 @@
    (_regs)->ints_state &= ~BIT(IC_RESTART); \
  } while (0)
 
-#define OFF_IC_BROADCAST(_regs) \
- do { \
-   (_regs)->ints_state &= ~BIT(IC_BROADCAST); \
- } while (0)
-
 #define OFF_IC_STORSTAT(_regs) \
  do { \
    (_regs)->ints_state &= ~BIT(IC_STORSTAT); \
@@ -552,37 +584,37 @@
 
 #define OFF_IC_PER(_regs) \
  do { \
-   (_regs)->ints_state &= ~IC_PER_MASK; \
+   (_regs)->ints_mask &= ~IC_PER_MASK; \
  } while (0)
 
 #define OFF_IC_PER_SB(_regs) \
  do { \
-   (_regs)->ints_state &= ~BIT(IC_PER_SB); \
+   (_regs)->ints_mask &= ~BIT(IC_PER_SB); \
  } while (0)
 
 #define OFF_IC_PER_IF(_regs) \
  do { \
-   (_regs)->ints_state &= ~BIT(IC_PER_IF); \
+   (_regs)->ints_mask &= ~BIT(IC_PER_IF); \
  } while (0)
 
 #define OFF_IC_PER_SA(_regs) \
  do { \
-   (_regs)->ints_state &= ~BIT(IC_PER_SA); \
+   (_regs)->ints_mask &= ~BIT(IC_PER_SA); \
  } while (0)
 
 #define OFF_IC_PER_GRA(_regs) \
  do { \
-   (_regs)->ints_state &= ~BIT(IC_PER_GRA); \
+   (_regs)->ints_mask &= ~BIT(IC_PER_GRA); \
  } while (0)
 
 #define OFF_IC_PER_STURA(_regs) \
  do { \
-   (_regs)->ints_state &= ~BIT(IC_PER_STURA); \
+   (_regs)->ints_mask &= ~BIT(IC_PER_STURA); \
  } while (0)
 
 #define OFF_IC_PER_IFNUL(_regs) \
  do { \
-   (_regs)->ints_state &= ~BIT(IC_PER_IFNUL); \
+   (_regs)->ints_mask &= ~BIT(IC_PER_IFNUL); \
  } while (0)
 
 
@@ -592,7 +624,6 @@
 
 #define IS_IC_INTERRUPT(_regs)  ( (_regs)->ints_state & BIT(IC_INTERRUPT) )
 #define IS_IC_RESTART(_regs)    ( (_regs)->ints_state & BIT(IC_RESTART)   )
-#define IS_IC_BROADCAST(_regs)  ( (_regs)->ints_state & BIT(IC_BROADCAST) )
 #define IS_IC_STORSTAT(_regs)   ( (_regs)->ints_state & BIT(IC_STORSTAT)  )
 #define IS_IC_IOPENDING         ( sysblk.ints_state   & BIT(IC_IO)        )
 #define IS_IC_MCKPENDING(_regs) ( (_regs)->ints_state &     IC_MCKPENDING )
@@ -606,13 +637,13 @@
 #define IS_IC_EXTCALL(_regs)    ( (_regs)->ints_state & BIT(IC_EXTCALL)   )
 #define IS_IC_MALFALT(_regs)    ( (_regs)->ints_state & BIT(IC_MALFALT)   )
 #define IS_IC_EMERSIG(_regs)    ( (_regs)->ints_state & BIT(IC_EMERSIG)   )
-#define IS_IC_PER(_regs)        ( (_regs)->ints_state &     IC_PER_MASK   )
-#define IS_IC_PER_SB(_regs)     ( (_regs)->ints_state & BIT(IC_PER_SB)    )
-#define IS_IC_PER_IF(_regs)     ( (_regs)->ints_state & BIT(IC_PER_IF)    )
-#define IS_IC_PER_SA(_regs)     ( (_regs)->ints_state & BIT(IC_PER_SA)    )
-#define IS_IC_PER_GRA(_regs)    ( (_regs)->ints_state & BIT(IC_PER_GRA)   )
-#define IS_IC_PER_STURA(_regs)  ( (_regs)->ints_state & BIT(IC_PER_STURA) )
-#define IS_IC_PER_IFNUL(_regs)  ( (_regs)->ints_state & BIT(IC_PER_IFNUL) )
+#define IS_IC_PER(_regs)        ( (_regs)->ints_mask  &     IC_PER_MASK   )
+#define IS_IC_PER_SB(_regs)     ( (_regs)->ints_mask  & BIT(IC_PER_SB)    )
+#define IS_IC_PER_IF(_regs)     ( (_regs)->ints_mask  & BIT(IC_PER_IF)    )
+#define IS_IC_PER_SA(_regs)     ( (_regs)->ints_mask  & BIT(IC_PER_SA)    )
+#define IS_IC_PER_GRA(_regs)    ( (_regs)->ints_mask  & BIT(IC_PER_GRA)   )
+#define IS_IC_PER_STURA(_regs)  ( (_regs)->ints_mask  & BIT(IC_PER_STURA) )
+#define IS_IC_PER_IFNUL(_regs)  ( (_regs)->ints_mask  & BIT(IC_PER_IFNUL) )
 
 
   /* * * * * * * * * * * * * *
@@ -628,12 +659,12 @@
    * * * * * * * * * * * * * */
 
 #define EN_IC_PER(_regs)        unlikely( (_regs)->permode )
-#define EN_IC_PER_SB(_regs)     ( EN_IC_PER(_regs) && ((_regs)->ints_mask & BIT(IC_PER_SB))    )
-#define EN_IC_PER_IF(_regs)     ( EN_IC_PER(_regs) && ((_regs)->ints_mask & BIT(IC_PER_IF))    )
-#define EN_IC_PER_SA(_regs)     ( EN_IC_PER(_regs) && ((_regs)->ints_mask & BIT(IC_PER_SA))    )
-#define EN_IC_PER_GRA(_regs)    ( EN_IC_PER(_regs) && ((_regs)->ints_mask & BIT(IC_PER_GRA))   )
-#define EN_IC_PER_STURA(_regs)  ( EN_IC_PER(_regs) && ((_regs)->ints_mask & BIT(IC_PER_STURA)) )
-#define EN_IC_PER_IFNUL(_regs)  ( EN_IC_PER(_regs) && ((_regs)->ints_mask & BIT(IC_PER_IFNUL)) )
+#define EN_IC_PER_SB(_regs)     ( EN_IC_PER(_regs) && ((_regs)->ints_state & BIT(IC_PER_SB))    )
+#define EN_IC_PER_IF(_regs)     ( EN_IC_PER(_regs) && ((_regs)->ints_state & BIT(IC_PER_IF))    )
+#define EN_IC_PER_SA(_regs)     ( EN_IC_PER(_regs) && ((_regs)->ints_state & BIT(IC_PER_SA))    )
+#define EN_IC_PER_GRA(_regs)    ( EN_IC_PER(_regs) && ((_regs)->ints_state & BIT(IC_PER_GRA))   )
+#define EN_IC_PER_STURA(_regs)  ( EN_IC_PER(_regs) && ((_regs)->ints_state & BIT(IC_PER_STURA)) )
+#define EN_IC_PER_IFNUL(_regs)  ( EN_IC_PER(_regs) && ((_regs)->ints_state & BIT(IC_PER_IFNUL)) )
 
 
   /* * * * * * * * * * * * * * * * * * * * * * * * *
@@ -679,19 +710,29 @@
 #define OPEN_IC_EMERSIG(_regs) \
                         ( (_regs)->ints_state & (_regs)->ints_mask & BIT(IC_EMERSIG) )
 
-#define OPEN_IC_PERINT(_regs) \
-                        ( (_regs)->ints_state                      & IC_PER_MASK)
-
+#define OPEN_IC_PER(_regs) \
+                        ( (_regs)->ints_state & (_regs)->ints_mask & IC_PER_MASK )
+#define OPEN_IC_PER_SB(_regs) \
+                        ( (_regs)->ints_state & (_regs)->ints_mask & BIT(IC_PER_SB) )
+#define OPEN_IC_PER_IF(_regs) \
+                        ( (_regs)->ints_state & (_regs)->ints_mask & BIT(IC_PER_IF) )
+#define OPEN_IC_PER_SA(_regs) \
+                        ( (_regs)->ints_state & (_regs)->ints_mask & BIT(IC_PER_SA) )
+#define OPEN_IC_PER_GRA(_regs) \
+                        ( (_regs)->ints_state & (_regs)->ints_mask & BIT(IC_PER_GRA) )
+#define OPEN_IC_PER_STURA(_regs) \
+                        ( (_regs)->ints_state & (_regs)->ints_mask & BIT(IC_PER_STURA) )
+#define OPEN_IC_PER_IFNUL(_regs) \
+                        ( (_regs)->ints_state & (_regs)->ints_mask & BIT(IC_PER_IFNUL) )
 
   /* * * * * * * * * * * * * * * * * * * * * * * * *
    * Check for general enabled pending interrupt   *
    * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #define IC_INTERRUPT_CPU(_regs) \
-   ( (_regs)->ints_state & ((_regs)->ints_mask | IC_PER_MASK) )
-
-#define IC_INTERRUPT_CPU_NO_PER(_regs) \
    ( (_regs)->ints_state & (_regs)->ints_mask )
+#define INTERRUPT_PENDING(_regs) IC_INTERRUPT_CPU((_regs))
 
 #define SIE_IC_INTERRUPT_CPU(_regs) \
-   (((_regs)->ints_state|((_regs)->hostregs->ints_state&IC_SIE_INT)) & ((_regs)->ints_mask|IC_PER_MASK))
+   (((_regs)->ints_state|((_regs)->hostregs->ints_state&IC_SIE_INT)) & (_regs)->ints_mask)
+#define SIE_INTERRUPT_PENDING(_regs) SIE_IC_INTERRUPT_CPU((_regs))
