@@ -1,6 +1,7 @@
-/* DYNINST.C    (c) Copyright Jan Jaeger, 2003-2006                  */
+/* DYNINST.C    (c) Copyright Jan Jaeger, 2003-2007                  */
 /*              Hercules Dynamic Loader                              */
 
+// $Id: dyninst.c,v 1.22 2007/06/23 00:04:09 ivan Exp $
 
 /* This module dynamically loads instructions.  Instruction routine  */
 /* names must be registered under the name of s370_opcode_B220 for   */
@@ -8,6 +9,13 @@
 /* mode respectively.  B220 is the opcode, and is depending on the   */
 /* instruction 2 3 or 4 digits.                                      */
 
+// $Log: dyninst.c,v $
+// Revision 1.22  2007/06/23 00:04:09  ivan
+// Update copyright notices to include current year (2007)
+//
+// Revision 1.21  2006/12/08 09:43:20  jj
+// Add CVS message log
+//
 
 #include "hstdinc.h"
 #include "hercules.h"
@@ -32,6 +40,7 @@
  #define opcode_b9xx  opcode_b9xx_r
  #define opcode_c0xx  opcode_c0xx_r
  #define opcode_c2xx  opcode_c2xx_r
+ #define opcode_c8xx  opcode_c8xx_r
  #define opcode_e3xx  opcode_e3xx_r
  #define opcode_e5xx  opcode_e5xx_r
  #define opcode_e6xx  opcode_e6xx_r
@@ -54,6 +63,7 @@
  #undef opcode_b9xx
  #undef opcode_c0xx
  #undef opcode_c2xx
+ #undef opcode_c8xx
  #undef opcode_e3xx
  #undef opcode_e5xx
  #undef opcode_e6xx
@@ -91,6 +101,7 @@ static zz_func save_b3xx[256][GEN_MAXARCH];
 static zz_func save_b9xx[256][GEN_MAXARCH];
 static zz_func save_c0xx[16][GEN_MAXARCH];
 static zz_func save_c2xx[16][GEN_MAXARCH];                      /*@Z9*/
+static zz_func save_c8xx[16][GEN_MAXARCH];
 static zz_func save_e3xx[256][GEN_MAXARCH];
 static zz_func save_e5xx[256][GEN_MAXARCH];
 static zz_func save_e6xx[256][GEN_MAXARCH];
@@ -113,6 +124,7 @@ static zz_func save_edxx[256][GEN_MAXARCH];
   static void * opcode_b9xx;
   static void * opcode_c0xx;
   static void * opcode_c2xx;                                    /*@Z9*/
+  static void * opcode_c8xx;
   static void * opcode_e3xx;
   static void * opcode_e5xx;
   static void * opcode_e6xx;
@@ -148,6 +160,7 @@ static void opcode_save()
     memcpy(save_b9xx,opcode_b9xx,sizeof(save_b9xx));
     memcpy(save_c0xx,opcode_c0xx,sizeof(save_c0xx));
     memcpy(save_c2xx,opcode_c2xx,sizeof(save_c2xx));            /*@Z9*/
+    memcpy(save_c8xx,opcode_c8xx,sizeof(save_c8xx));
     memcpy(save_e3xx,opcode_e3xx,sizeof(save_e3xx));
     memcpy(save_e5xx,opcode_e5xx,sizeof(save_e5xx));
     memcpy(save_e6xx,opcode_e6xx,sizeof(save_e6xx));
@@ -171,6 +184,7 @@ static void opcode_restore()
     memcpy(opcode_b9xx,save_b9xx,sizeof(save_b9xx));
     memcpy(opcode_c0xx,save_c0xx,sizeof(save_c0xx));
     memcpy(opcode_c2xx,save_c2xx,sizeof(save_c2xx));            /*@Z9*/
+    memcpy(opcode_c8xx,save_c2xx,sizeof(save_c8xx));
     memcpy(opcode_e3xx,save_e3xx,sizeof(save_e3xx));
     memcpy(opcode_e5xx,save_e5xx,sizeof(save_e5xx));
     memcpy(opcode_e6xx,save_e6xx,sizeof(save_e6xx));
@@ -272,7 +286,7 @@ HDL_DEPENDENCY_SECTION;
      HDL_DEPENDENCY (DEVBLK);
      HDL_DEPENDENCY (SYSBLK);
 
-} END_DEPENDENCY_SECTION;
+} END_DEPENDENCY_SECTION
 
 
 HDL_REGISTER_SECTION;
@@ -283,7 +297,7 @@ HDL_REGISTER_SECTION;
     opcode_save();
 #endif
 }
-END_REGISTER_SECTION;
+END_REGISTER_SECTION
 
 
 HDL_RESOLVER_SECTION;
@@ -307,6 +321,7 @@ int opcode, extop;
         HDL_RESOLVE(opcode_b9xx);
         HDL_RESOLVE(opcode_c0xx);
         HDL_RESOLVE(opcode_c2xx);                               /*@Z9*/
+        HDL_RESOLVE(opcode_c8xx);
         HDL_RESOLVE(opcode_e3xx);
         HDL_RESOLVE(opcode_e5xx);
         HDL_RESOLVE(opcode_e6xx);
@@ -372,6 +387,11 @@ int opcode, extop;
                     assign_extop1(opcode, extop, opcode_c2xx, save_c2xx);  /*@Z9*/
                 break;                                                     /*@Z9*/
 
+            case 0xC8:
+                for(extop = 0; extop < 16; extop++)
+                    assign_extop1(opcode, extop, opcode_c8xx, save_c8xx);
+                break;
+
             case 0xE3:
                 for(extop = 0; extop < 256; extop++)
                     assign_extop(opcode, extop, opcode_e3xx, save_e3xx);
@@ -411,7 +431,7 @@ int opcode, extop;
     /* Copy opcodes to performance shadow tables */
     copy_opcode_tables();
 
-} END_RESOLVER_SECTION;
+} END_RESOLVER_SECTION
 
 
 HDL_FINAL_SECTION;
@@ -419,7 +439,7 @@ HDL_FINAL_SECTION;
 
     opcode_restore();
 
-} END_FINAL_SECTION;
+} END_FINAL_SECTION
 
 
 #endif /*!defined(_GEN_ARCH)*/
