@@ -1,10 +1,26 @@
 ////////////////////////////////////////////////////////////////////////////////////
 //         fishhang.c           verify/debug proper Hercules LOCK handling...
 ////////////////////////////////////////////////////////////////////////////////////
-// (c) Copyright "Fish" (David B. Trout), 2002-2006. Released under the Q Public License
+// (c) Copyright "Fish" (David B. Trout), 2002-2007. Released under the Q Public License
 // (http://www.conmicro.cx/hercules/herclic.html) as modifications to Hercules.
 ////////////////////////////////////////////////////////////////////////////////////
 
+// $Id: fishhang.c,v 1.18 2007/06/23 00:04:09 ivan Exp $
+//
+// $Log: fishhang.c,v $
+// Revision 1.18  2007/06/23 00:04:09  ivan
+// Update copyright notices to include current year (2007)
+//
+// Revision 1.17  2006/12/31 12:25:26  fish
+// Fix "undefined reference to _beginthreadex" issue for non-MSVC (i.e. Cygwin) builds.
+//
+// Revision 1.16  2006/12/28 15:49:34  fish
+// Use _beginthreadex/_endthreadex instead of CreateThread/ExitThread in continuing effort to try and resolve our still existing long-standing 'errno' issue...
+//
+// Revision 1.15  2006/12/08 09:43:21  jj
+// Add CVS message log
+//
+
 #include "hstdinc.h"
 
 #define _FISHHANG_C_
@@ -707,7 +723,11 @@ HANDLE FishHang_CreateThread
 
     if (!(pFISH_THREAD = CreateFISH_THREAD(pszFileCreated,nLineCreated))) return NULL;
 
+#ifdef _MSVC_
+    hThread = (HANDLE) _beginthreadex
+#else // (Cygwin)
     hThread = CreateThread
+#endif
     (
         lpThreadAttributes, // pointer to security attributes
         dwStackSize,        // initial thread stack size
@@ -1505,7 +1525,7 @@ void FishHang_ExitThread
     {
         UnlockFishHang();
         logmsg("** FishHang_ExitThread: FindFISH_THREAD failed!\n");
-        if (bFishHangAtExit) ExitThread(dwExitCode);
+        if (bFishHangAtExit) _endthreadex(dwExitCode);
         exit(-1);
     }
 
@@ -1535,7 +1555,7 @@ void FishHang_ExitThread
 
     UnlockFishHang();
 
-    ExitThread(dwExitCode);
+    _endthreadex(dwExitCode);
 }
 
 /////////////////////////////////////////////////////////////////////////////
