@@ -1,8 +1,18 @@
 /*********************************************************************/
-/* HDL.H        (c) Copyright Jan Jaeger, 2003-2006                  */
+/* HDL.H        (c) Copyright Jan Jaeger, 2003-2007                  */
 /*              Hercules Dynamic Loader                              */
 /*********************************************************************/
 
+// $Id: hdl.h,v 1.34 2007/06/23 00:04:10 ivan Exp $
+//
+// $Log: hdl.h,v $
+// Revision 1.34  2007/06/23 00:04:10  ivan
+// Update copyright notices to include current year (2007)
+//
+// Revision 1.33  2006/12/08 09:43:25  jj
+// Add CVS message log
+//
+
 #ifndef _HDL_H
 #define _HDL_H
 
@@ -50,14 +60,14 @@ typedef struct _HDLSHD {
 HDL_DLL_IMPORT void    hdl_adsc(char*, void *, void *);/* Add shutdown routine      */
 HDL_DLL_IMPORT int     hdl_rmsc(void *, void *);       /* Remove shutdown routine   */
 HDL_DLL_IMPORT void    hdl_shut(void);                 /* Call all shutdown routines*/
-DLL_EXPORT DEVHND *hdl_ghnd(char *devname);        /* Get device handler        */
+DLL_EXPORT DEVHND *hdl_ghnd(const char *devname);        /* Get device handler        */
 
 /*********************************************************************/
 
 #if !defined(OPTION_DYNAMIC_LOAD)
 
 #define HDL_DEVICE_SECTION                              \
-DLL_EXPORT DEVHND *hdl_ghnd(char *devtype)                         \
+DLL_EXPORT DEVHND *hdl_ghnd(const char *devtype)                         \
 {
 
 #define HDL_DEVICE( _devname, _devhnd )                 \
