@@ -1,7 +1,7 @@
 /*
 || ----------------------------------------------------------------------------
 ||
-|| HETGET.C     (c) Copyright Leland Lucius, 2000-2006
+|| HETGET.C     (c) Copyright Leland Lucius, 2000-2007
 ||              Released under terms of the Q Public License.
 ||
 || Extract files from an HET file
@@ -9,6 +9,16 @@
 || ----------------------------------------------------------------------------
 */
 
+// $Id: hetget.c,v 1.25 2007/06/23 00:04:10 ivan Exp $
+//
+// $Log: hetget.c,v $
+// Revision 1.25  2007/06/23 00:04:10  ivan
+// Update copyright notices to include current year (2007)
+//
+// Revision 1.24  2006/12/08 09:43:26  jj
+// Add CVS message log
+//
+
 #include "hstdinc.h"
 
 #include "hercules.h"
@@ -117,7 +127,7 @@ int reclen = 0;
 /*
 || Previously reported file position
 */
-static OFF_T prevpos = 0;
+static off_t prevpos = 0;
 /*
 || Report progress every this many bytes
 */
@@ -505,7 +515,7 @@ getfile( HETB *hetb, FILE *outf )
             if( extgui )
             {
                 /* Report progress every nnnK */
-                OFF_T curpos = FTELL( hetb->fd );
+                off_t curpos = ftell( hetb->fd );
                 if( ( curpos & PROGRESS_MASK ) != ( prevpos & PROGRESS_MASK ) )
                 {
                     prevpos = curpos;
@@ -571,7 +581,7 @@ getfile( HETB *hetb, FILE *outf )
             if( extgui )
             {
                 /* Report progress every nnnK */
-                OFF_T curpos = FTELL( hetb->fd );
+                off_t curpos = ftell( hetb->fd );
                 if( ( curpos & PROGRESS_MASK ) != ( prevpos & PROGRESS_MASK ) )
                 {
                     prevpos = curpos;
