@@ -1,7 +1,7 @@
 /*
 || ----------------------------------------------------------------------------
 ||
-|| HETMAP.C     (c) Copyright Leland Lucius, 2000-2006
+|| HETMAP.C     (c) Copyright Leland Lucius, 2000-2007
 ||              Released under terms of the Q Public License.
 ||
 || Displays information about the structure of a Hercules Emulated Tape.
@@ -9,6 +9,16 @@
 || ----------------------------------------------------------------------------
 */
 
+// $Id: hetmap.c,v 1.26 2007/06/23 00:04:11 ivan Exp $
+//
+// $Log: hetmap.c,v $
+// Revision 1.26  2007/06/23 00:04:11  ivan
+// Update copyright notices to include current year (2007)
+//
+// Revision 1.25  2006/12/08 09:43:26  jj
+// Add CVS message log
+//
+
 #include "hstdinc.h"
 
 #include "hercules.h"
@@ -32,7 +42,7 @@ static const char help[] =
 
 #ifdef EXTERNALGUI
 /* Previous reported file position */
-static OFF_T prevpos = 0;
+static off_t prevpos = 0;
 /* Report progress every this many bytes */
 #define PROGRESS_MASK (~0x3FFFF /* 256K */)
 #endif /*EXTERNALGUI*/
@@ -247,7 +257,7 @@ main( int argc, char *argv[] )
         if( extgui )
         {
             /* Report progress every nnnK */
-            OFF_T curpos = FTELL( hetb->fd );
+            off_t curpos = ftell( hetb->fd );
             if( ( curpos & PROGRESS_MASK ) != ( prevpos & PROGRESS_MASK ) )
             {
                 prevpos = curpos;
