@@ -1,3 +1,10 @@
+// $Id: pttrace.h,v 1.15 2006/12/08 09:43:29 jj Exp $
+//
+// $Log: pttrace.h,v $
+// Revision 1.15  2006/12/08 09:43:29  jj
+// Add CVS message log
+//
+
 #if !defined( _PTTHREAD_H_ )
 #define _PTTHREAD_H_
 
@@ -63,7 +70,8 @@ PTT_DLL_IMPORT int ptt_pthread_kill(TID, int, char *, int);
 PTT_DLL_IMPORT void ptt_trace_init (int n, int init);
 PTT_DLL_IMPORT int  ptt_cmd(int argc, char *argv[], char*cmdline);
 PTT_DLL_IMPORT void ptt_pthread_trace (char *, void *, void *, char *, int, int);
-PTT_DLL_IMPORT void ptt_pthread_print ();
+PTT_DLL_IMPORT int  ptt_pthread_print ();
+void *ptt_timeout();
 
 typedef struct _PTT_TRACE {
         TID          tid;               /* Thead id                   */
