@@ -1,8 +1,10 @@
-/* TRACE.C      (c) Copyright Jan Jaeger, 2000-2006                  */
+/* TRACE.C      (c) Copyright Jan Jaeger, 2000-2007                  */
 /*              Implicit tracing functions                           */
 
-/* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2006      */
-/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2006      */
+// $Id: trace.c,v 1.31 2007/06/23 00:04:19 ivan Exp $
+
+/* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2007      */
+/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2007      */
 
 /*-------------------------------------------------------------------*/
 /* This module contains procedures for creating entries in the       */
@@ -16,6 +18,17 @@
 /*      ASN-and-LX-reuse facility - Roger Bowler            July 2004*/
 /*-------------------------------------------------------------------*/
 
+// $Log: trace.c,v $
+// Revision 1.31  2007/06/23 00:04:19  ivan
+// Update copyright notices to include current year (2007)
+//
+// Revision 1.30  2006/12/20 04:26:20  gsmith
+// 19 Dec 2006 ip_all.pat - performance patch - Greg Smith
+//
+// Revision 1.29  2006/12/08 09:43:31  jj
+// Add CVS message log
+//
+
 #include "hstdinc.h"
 
 #if !defined(_HENGINE_DLL_)
@@ -605,7 +618,7 @@ RADR ag;
 int  size;
 int  eamode;
 
-    regs->psw.IA &= ADDRESS_MAXWRAP(regs);
+    SET_PSW_IA(regs);
     eamode = regs->psw.amode64;
 
 #if defined(FEATURE_ESAME)
@@ -741,7 +754,8 @@ RADR raddr;
 RADR ag;
 int  size;
 
-    regs->psw.IA &= ADDRESS_MAXWRAP(regs);
+    SET_PSW_IA(regs);
+    SET_PSW_IA(newregs);
 
 #if defined(FEATURE_ESAME)
     if(!regs->psw.amode64 && !newregs->psw.amode64)
@@ -962,7 +976,7 @@ RADR raddr;
 RADR ag;
 int  size;
 
-    regs->psw.IA &= ADDRESS_MAXWRAP(regs);
+    SET_PSW_IA(regs);
 
     if(!br)
     {
