@@ -2,7 +2,7 @@
 // Hercules - TUN/TAP Abstraction Layer
 // ====================================================================
 //
-// Copyright (C) 2002-2006 by James A. Pierson
+// Copyright (C) 2002-2007 by James A. Pierson
 //           (C) 2002-2006 by "Fish" (David B. Trout)
 //
 // TUN/TAP implementations differ among platforms. Linux and FreeBSD
@@ -15,6 +15,16 @@
 // that works on all platforms with (hopefully) equal results.
 //
 
+// $Id: tuntap.c,v 1.45 2007/06/23 00:04:19 ivan Exp $
+//
+// $Log: tuntap.c,v $
+// Revision 1.45  2007/06/23 00:04:19  ivan
+// Update copyright notices to include current year (2007)
+//
+// Revision 1.44  2006/12/08 09:43:31  jj
+// Add CVS message log
+//
+
 #include "hstdinc.h"
 
 #include "hercules.h"
@@ -51,6 +61,93 @@ static void tuntap_term(void)
 // Primary Module Entry Points
 // ====================================================================
 
+static int TUNTAP_SetMode (int fd, struct ifreq *ifr)
+{
+    int rc;
+
+    /* Try TUNTAP_ioctl first */
+    rc = TUNTAP_IOCtl (fd, TUNSETIFF, (char *) ifr);
+
+#if !defined(OPTION_W32_CTCI)
+    /* If invalid value, try with the pre-2.4.5 value */
+    if (rc != 0 && errno == EINVAL)
+        rc = TUNTAP_IOCtl (fd, ('T' << 8) | 202, (char *) ifr);
+
+    /* kludge for EPERM and linux 2.6.18 */
+    if (rc != 0 && errno == EPERM)
+    {
+        int             ifd[2];
+        char           *hercifc;
+        pid_t           pid;
+        CTLREQ          ctlreq;
+        fd_set          selset;
+        struct timeval  tv;
+        int             sv_err;
+        int             status;
+
+        if (socketpair (AF_UNIX, SOCK_STREAM, 0, ifd) < 0)
+            return -1;
+
+        if (!(hercifc = getenv ("HERCULES_IFC")))
+            hercifc = HERCIFC_CMD;
+
+        pid = fork();
+
+        if (pid < 0)
+            return -1;
+        else if (pid == 0)
+        {
+            /* child */
+            dup2 (ifd[0], STDIN_FILENO);
+            dup2 (STDOUT_FILENO, STDERR_FILENO);
+            dup2 (ifd[0], STDOUT_FILENO);
+            close (ifd[1]);
+            rc = execlp (hercifc, hercifc, NULL );
+            return -1;
+        }
+
+        /* parent */
+        close(ifd[0]);
+
+        /* Request hercifc to issue the TUNSETIFF ioctl */
+        memset (&ctlreq, 0, CTLREQ_SIZE);
+        ctlreq.iCtlOp = TUNSETIFF;
+        ctlreq.iProcID = fd;
+        memcpy (&ctlreq.iru.ifreq, ifr, sizeof (struct ifreq));
+        write (ifd[1], &ctlreq, CTLREQ_SIZE);
+
+        /* Get response, if any, from hercifc */
+        FD_ZERO (&selset);
+        FD_SET (ifd[1], &selset);
+        tv.tv_sec = 5;
+        tv.tv_usec = 0;
+        rc = select (ifd[1]+1, &selset, NULL, NULL, &tv);
+        if (rc > 0)
+        {
+            rc = read (ifd[1], &ctlreq, CTLREQ_SIZE);
+            if (rc > 0)
+                memcpy (ifr, &ctlreq.iru.ifreq, sizeof (struct ifreq));
+        }
+        else if (rc == 0)
+        {
+            logmsg (_("HHCTU001E %s timeout, possible older version?\n"),
+                    hercifc);
+            errno = EPERM;
+            rc = -1;
+        }
+
+        /* clean-up */
+        sv_err = errno;
+        close (ifd[1]);
+        kill (pid, SIGINT);
+        waitpid (pid, &status, 0);
+        errno = sv_err;
+    }
+#endif /* if !defined(OPTION_W32_CTCI) */
+
+    return rc;
+}
+
 //
 // TUNTAP_CreateInterface
 //
@@ -98,7 +195,7 @@ static void tuntap_term(void)
 //                       IFF_NO_PI - Do not include packet information
 //
 // Output:
-//      pfd           Pointer to receive the file descriptor if the
+//      pfd           Pointer to receive the file descriptor of the
 //                       TUN/TAP interface.
 //      pszNetDevName Pointer to receive the name if the interface.
 //
@@ -142,14 +239,9 @@ int             TUNTAP_CreateInterface( char* pszTUNDevice,
         struct ifreq ifr;
 
         memset( &ifr, 0, sizeof( ifr ) );
-
         ifr.ifr_flags = iFlags;
 
-        // First try the value from the header that we ship (2.4.8)
-        // If this fails with EINVAL, try with the pre-2.4.5 value
-        if( TUNTAP_IOCtl( fd, TUNSETIFF, (char*)&ifr ) != 0 &&
-            ( errno != EINVAL ||
-              TUNTAP_IOCtl( fd, ('T' << 8) | 202, (char*)&ifr ) != 0 )  )
+        if( TUNTAP_SetMode (fd, &ifr) < 0 )
         {
             logmsg( _("HHCTU003E Error setting TUN/TAP mode: %s: %s\n"),
                     pszTUNDevice, strerror( errno ) );
@@ -196,6 +288,30 @@ int             TUNTAP_CreateInterface( char* pszTUNDevice,
   #define TUNTAP_IOCtl    IFC_IOCtl
 #endif
 
+#ifdef   OPTION_TUNTAP_CLRIPADDR
+//
+// TUNTAP_ClrIPAddr
+//
+
+int             TUNTAP_ClrIPAddr( char*   pszNetDevName )
+{
+    struct ifreq        ifreq;
+
+    memset( &ifreq, 0, sizeof( struct ifreq ) );
+
+    if( !pszNetDevName || !*pszNetDevName )
+    {
+        logmsg( _("HHCTU005E Invalid net device name specified: %s\n"),
+                pszNetDevName ? pszNetDevName : "(null pointer)" );
+        return -1;
+    }
+
+    strcpy( ifreq.ifr_name, pszNetDevName );
+
+    return TUNTAP_IOCtl( 0, SIOCDIFADDR, (char*)&ifreq );
+}
+#endif /* OPTION_TUNTAP_CLRIPADDR */
+
 //
 // TUNTAP_SetIPAddr
 //
@@ -211,6 +327,7 @@ int             TUNTAP_SetIPAddr( char*   pszNetDevName,
     sin = (struct sockaddr_in*)&ifreq.ifr_addr;
 
     sin->sin_family = AF_INET;
+    set_sockaddr_in_sin_len( sin );
 
     if( !pszNetDevName || !*pszNetDevName )
     {
@@ -247,6 +364,7 @@ int             TUNTAP_SetDestAddr( char*   pszNetDevName,
     sin = (struct sockaddr_in*)&ifreq.ifr_addr;
 
     sin->sin_family = AF_INET;
+    set_sockaddr_in_sin_len( sin );
 
     if( !pszNetDevName || !*pszNetDevName )
     {
@@ -283,6 +401,7 @@ int           TUNTAP_SetNetMask( char*   pszNetDevName,
     sin = (struct sockaddr_in*)&ifreq.ifr_netmask;
 
     sin->sin_family = AF_INET;
+    set_sockaddr_in_sin_len( sin );
 
     if( !pszNetDevName || !*pszNetDevName )
     {
@@ -320,6 +439,7 @@ int             TUNTAP_SetMTU( char*   pszNetDevName,
     sin = (struct sockaddr_in*)&ifreq.ifr_addr;
 
     sin->sin_family = AF_INET;
+    set_sockaddr_in_sin_len( sin );
 
     if( !pszNetDevName || !*pszNetDevName )
     {
@@ -405,6 +525,7 @@ int             TUNTAP_SetFlags ( char*   pszNetDevName,
     sin = (struct sockaddr_in*)&ifreq.ifr_addr;
 
     sin->sin_family = AF_INET;
+    set_sockaddr_in_sin_len( sin );
 
     if( !pszNetDevName || !*pszNetDevName )
     {
@@ -413,13 +534,65 @@ int             TUNTAP_SetFlags ( char*   pszNetDevName,
         return -1;
     }
 
-    strcpy( ifreq.ifr_name, pszNetDevName );
+    strlcpy( ifreq.ifr_name, pszNetDevName, sizeof(ifreq.ifr_name) );
 
     ifreq.ifr_flags = iFlags;
 
     return TUNTAP_IOCtl( 0, SIOCSIFFLAGS, (char*)&ifreq );
 }
 
+//
+// TUNTAP_GetFlags
+//
+
+int      TUNTAP_GetFlags ( char*   pszNetDevName,
+                           int*    piFlags )
+{
+    struct ifreq        ifreq;
+    struct sockaddr_in* sin;
+    int                 rc;
+
+    memset( &ifreq, 0, sizeof( struct ifreq ) );
+
+    sin = (struct sockaddr_in*)&ifreq.ifr_addr;
+
+    sin->sin_family = AF_INET;
+
+    if( !pszNetDevName || !*pszNetDevName )
+    {
+        logmsg( _("HHCTU016E Invalid net device name specified: %s\n"),
+                pszNetDevName ? pszNetDevName : "(null pointer)" );
+        return -1;
+    }
+
+    strlcpy( ifreq.ifr_name, pszNetDevName, sizeof(ifreq.ifr_name) );
+
+    // PROGRAMMING NOTE: hercifc can't "get" information,
+    // only "set" it. Thus because we normally use hercifc
+    // to issue ioctl codes to the interface (on non-Win32)
+    // we bypass hercifc altogether and issue the ioctl
+    // ourselves directly to the device itself, bypassing
+    // hercifc completely. Note that for Win32 however,
+    // 'TUNTAP_IOCtl' routes to a TunTap32.DLL call and
+    // thus works just fine. We need special handling
+    // only for non-Win32 platforms. - Fish
+
+#if defined( OPTION_W32_CTCI )
+
+    rc = TUNTAP_IOCtl( 0, SIOCGIFFLAGS, (char*)&ifreq );
+
+#else // (non-Win32 platforms)
+    {
+        int sockfd = socket( AF_INET, SOCK_DGRAM, 0 );
+        rc = ioctl( sockfd, SIOCGIFFLAGS, &ifreq );
+    }
+#endif
+
+    *piFlags = ifreq.ifr_flags;
+
+    return rc;
+}
+
 //
 // TUNTAP_AddRoute
 //
@@ -446,6 +619,7 @@ int           TUNTAP_AddRoute( char*   pszNetDevName,
 
     sin = (struct sockaddr_in*)&rtentry.rt_dst;
     sin->sin_family = AF_INET;
+    set_sockaddr_in_sin_len( sin );
 
     if( !pszDestAddr  ||
         !inet_aton( pszDestAddr, &sin->sin_addr ) )
@@ -457,6 +631,7 @@ int           TUNTAP_AddRoute( char*   pszNetDevName,
 
     sin = (struct sockaddr_in*)&rtentry.rt_genmask;
     sin->sin_family = AF_INET;
+    set_sockaddr_in_sin_len( sin );
 
     if( !pszNetMask  ||
         !inet_aton( pszNetMask, &sin->sin_addr ) )
@@ -466,15 +641,16 @@ int           TUNTAP_AddRoute( char*   pszNetDevName,
         return -1;
     }
 
+    sin = (struct sockaddr_in*)&rtentry.rt_gateway;
+    sin->sin_family = AF_INET;
+    set_sockaddr_in_sin_len( sin );
+
     if( pszGWAddr )
     {
-        sin = (struct sockaddr_in*)&rtentry.rt_gateway;
-        sin->sin_family = AF_INET;
-
         if( !inet_aton( pszGWAddr, &sin->sin_addr ) )
         {
             logmsg( _("HHCTU020E %s: Invalid gateway address: %s.\n"),
-                    pszNetDevName, !pszGWAddr ? "NULL" : pszGWAddr );
+                    pszNetDevName, pszGWAddr );
             return -1;
         }
     }
@@ -511,6 +687,7 @@ int           TUNTAP_DelRoute( char*   pszNetDevName,
 
     sin = (struct sockaddr_in*)&rtentry.rt_dst;
     sin->sin_family = AF_INET;
+    set_sockaddr_in_sin_len( sin );
 
     if( !pszDestAddr  ||
         !inet_aton( pszDestAddr, &sin->sin_addr ) )
@@ -522,6 +699,7 @@ int           TUNTAP_DelRoute( char*   pszNetDevName,
 
     sin = (struct sockaddr_in*)&rtentry.rt_genmask;
     sin->sin_family = AF_INET;
+    set_sockaddr_in_sin_len( sin );
 
     if( !pszNetMask  ||
         !inet_aton( pszNetMask, &sin->sin_addr ) )
@@ -533,13 +711,16 @@ int           TUNTAP_DelRoute( char*   pszNetDevName,
 
     sin = (struct sockaddr_in*)&rtentry.rt_gateway;
     sin->sin_family = AF_INET;
+    set_sockaddr_in_sin_len( sin );
 
-    if( !pszGWAddr  ||
-        !inet_aton( pszGWAddr, &sin->sin_addr ) )
+    if( pszGWAddr )
     {
-        logmsg( _("HHCTU024E %s: Invalid gateway address: %s.\n"),
-                pszNetDevName, !pszGWAddr ? "NULL" : pszGWAddr );
-        return -1;
+        if( !inet_aton( pszGWAddr, &sin->sin_addr ) )
+        {
+            logmsg( _("HHCTU024E %s: Invalid gateway address: %s.\n"),
+                    pszNetDevName, pszGWAddr );
+            return -1;
+        }
     }
 
     rtentry.rt_flags = iFlags;
@@ -560,10 +741,12 @@ int           TUNTAP_DelRoute( char*   pszNetDevName,
 static int      IFC_IOCtl( int fd, unsigned long int iRequest, char* argp )
 {
     char*       pszCfgCmd;     // Interface config command
-    /* char*       request_name; */ // debugging: name of ioctl request
-    /* char        unknown_request[]="Unknown (0x00000000)"; */
     int         rc;
     CTLREQ      ctlreq;
+    char*       request_name;  // debugging: name of ioctl request
+#if defined(DEBUG) || defined(_DEBUG)
+    char        unknown_request[] = "Unknown (0x00000000)";
+#endif
 
     UNREFERENCED( fd );
 
@@ -571,40 +754,51 @@ static int      IFC_IOCtl( int fd, unsigned long int iRequest, char* argp )
 
     ctlreq.iCtlOp = iRequest;
 
-#if 0 /* ++++++++++++++++++++++ debugging print ++++++++++++++++++++++ */
+#if defined(DEBUG) || defined(_DEBUG)
 
     // Select string to represent ioctl request for debugging.
+
     switch (iRequest) {
-    case SIOCSIFADDR:
+#ifdef OPTION_TUNTAP_CLRIPADDR
+    case              SIOCDIFADDR:
+        request_name="SIOCDIFADDR"; break;
+#endif
+    case              SIOCSIFADDR:
         request_name="SIOCSIFADDR"; break;
-    case SIOCSIFDSTADDR:
+
+    case              SIOCSIFDSTADDR:
         request_name="SIOCSIFDSTADDR"; break;
-    case SIOCSIFMTU:
+
+    case              SIOCSIFMTU:
         request_name="SIOCSIFMTU"; break;
-    case SIOCSIFFLAGS:
+
+    case              SIOCSIFFLAGS:
         request_name="SIOCSIFFLAGS"; break;
+
+    case              SIOCGIFFLAGS:
+        request_name="SIOCGIFFLAGS"; break;
+
 #ifdef OPTION_TUNTAP_SETNETMASK
-    case SIOCSIFNETMASK:
+    case              SIOCSIFNETMASK:
         request_name="SIOCSIFNETMASK"; break;
 #endif
 #ifdef OPTION_TUNTAP_SETMACADDR
-    case SIOCSIFHWADDR:
+    case              SIOCSIFHWADDR:
         request_name="SIOCSIFHWADDR"; break;
 #endif
 #ifdef OPTION_TUNTAP_DELADD_ROUTES
-    case SIOCADDRT:
+    case              SIOCADDRT:
         request_name="SIOCADDRT"; break;
-    case SIOCDELRT:
+
+    case              SIOCDELRT:
         request_name="SIOCDELRT"; break;
 #endif
     default:
         sprintf(unknown_request,"Unknown (0x%x)",iRequest);
         request_name=unknown_request;
     }
-logmsg(_("HHCTU030I IFC_IOCtl called for %s on FDs %d %d\n"),
-          request_name,ifc_fd[0],ifc_fd[1]);
 
-#endif /* ++++++++++++++++++++++ debugging print ++++++++++++++++++++++ */
+#endif // defined(DEBUG) || defined(_DEBUG)
 
 #ifdef OPTION_TUNTAP_DELADD_ROUTES
     if( iRequest == SIOCADDRT ||
@@ -632,8 +826,9 @@ logmsg(_("HHCTU030I IFC_IOCtl called for %s on FDs %d %d\n"),
         // Obtain the name of the interface config program or default
         if( !( pszCfgCmd = getenv( "HERCULES_IFC" ) ) )
             pszCfgCmd = HERCIFC_CMD;
-//DEBUG     logmsg(_("HHCTU029I Executing '%s' to configure interface\n")
-//DEBUG               pszCfgCmd);
+
+        TRACE(_("HHCTU029I Executing '%s' to configure interface\n"),
+            pszCfgCmd);
 
         // Fork a process to execute the hercifc
         ifc_pid = fork();
@@ -663,8 +858,10 @@ logmsg(_("HHCTU030I IFC_IOCtl called for %s on FDs %d %d\n"),
              * billion files. -- JRM */
             file_limit=rlim.rlim_max;
             file_limit=(file_limit>1024)?1024:file_limit;
-//DEBUG     logmsg(_("HHCTU031I Closing %" I64_FMT "d files\n"),
-//DEBUG              (long long)file_limit);
+
+            TRACE(_("HHCTU031I Closing %" I64_FMT "d files\n"),
+                (long long)file_limit);
+
             for(i=0;(unsigned int)i<file_limit;i++)
             {
                 if(i!=ifc_fd[1] && i!=STDOUT_FILENO)
@@ -693,9 +890,94 @@ logmsg(_("HHCTU030I IFC_IOCtl called for %s on FDs %d %d\n"),
     // Populate some common fields
     ctlreq.iType = 1;
 
+    TRACE(_("HHCTU030I IFC_IOCtl called for %s on FDs %d %d\n"),
+        request_name,ifc_fd[0],ifc_fd[1]);
+
     write( ifc_fd[0], &ctlreq, CTLREQ_SIZE );
 
     return 0;
 }
 
 #endif // !defined( OPTION_W32_CTCI )
+
+// The following function used by Win32 *and* NON-Win32 platforms...
+
+void build_herc_iface_mac ( BYTE* out_mac, const BYTE* in_ip )
+{
+    // Routine to build a default MAC address for the CTCI device's
+    // virtual interface... (used by ctc_ctci.c CTCI_Init function)
+
+    if (!in_ip || !out_mac)
+    {
+        ASSERT( FALSE );
+        return;                 // (nothing for us to do!)
+    }
+
+#if defined( OPTION_W32_CTCI )
+
+    // We prefer to let TunTap32 do it for us (since IT'S the one
+    // that decides what it should really be) but if they're using
+    // an older version of TunTap32 that doesn't have the function
+    // then we'll do it ourselves just like before...
+
+    if (tt32_build_herc_iface_mac( out_mac, in_ip ))
+        return;
+
+#endif
+
+    // Build a default MAC addr based on the guest (destination) ip
+    // address so as to effectively *UNOFFICIALLY* assign ourselves
+    // the following Ethernet address block:
+
+    /* (from: http://www.iana.org/assignments/ethernet-numbers)
+       (only the first 2 and last 2 paragraphs are of interest)
+
+        IANA ETHERNET ADDRESS BLOCK - UNICAST USE
+
+        The IANA owns an Ethernet address block which may be used for
+        unicast address asignments or other special purposes.
+
+        The IANA may assign unicast global IEEE 802 MAC address from it's
+        assigned OUI (00-00-5E) for use in IETF standard track protocols.  The
+        intended usage is for dynamic mapping between IP addresses and IEEE
+        802 MAC addresses.  These IEEE 802 MAC addresses are not to be
+        permanently assigned to any hardware interface, nor is this a
+        substitute for a network equipment supplier getting its own OUI.
+
+        ... (snipped)
+
+        Using this representation, the range of Internet Unicast addresses is:
+
+               00-00-5E-00-00-00  to  00-00-5E-FF-FF-FF  in hex, ...
+
+        ... (snipped)
+
+        The low order 24 bits of these unicast addresses are assigned as
+        follows:
+
+        Dotted Decimal          Description                     Reference
+        ----------------------- ------------------------------- ---------
+        000.000.000-000.000.255 Reserved                        [IANA]
+        000.001.000-000.001.255 Virual Router Redundancy (VRRP) [Hinden]
+        000.002.000-127.255.255 Reserved                        [IANA]
+        128.000.000-255.255.255 Hercules TUNTAP (CTCI)          [Fish] (*UNOFFICIAL*)
+    */
+
+    // Here's what we're basically doing:
+
+    //    00-00-5E-00-00-00  to  00-00-5E-00-00-FF  =  'Reserved' by IANA
+    //    00-00-5E-00-01-00  to  00-00-5E-00-01-FF  =  'VRRP' by Hinden
+    //    00-00-5E-00-02-00  to  00-00-5E-7F-FF-FF  =  (unassigned)
+    //    00-00-5E-80-00-00  to  00-00-5E-FF-FF-FF  =  'Hercules' by Fish (*UNOFFICIAL*)
+
+    //    00-00-5E-00-00-00   (starting value)
+    //    00-00-5E-ip-ip-ip   (move in low-order 3 bytes of destination IP address)
+    //    00-00-5E-8p-ip-ip   ('OR' on the x'80' high-order bit)
+
+    *(out_mac+0) = 0x00;
+    *(out_mac+1) = 0x00;
+    *(out_mac+2) = 0x5E;
+    *(out_mac+3) = *(in_ip+1) | 0x80;
+    *(out_mac+4) = *(in_ip+2);
+    *(out_mac+5) = *(in_ip+3);
+}
